Imports System.Configuration
Imports Microsoft.AccessControl2.SDK

Namespace Azure.AccessControl.SecurityModule
	Friend Class SWTModule
		Implements IHttpModule

		'service namesapce setup in azure
		Private _serviceNamespace As String = ConfigurationManager.AppSettings("serviceNamespace")

		'azure access control service name...
		Private _acsHostName As String = ConfigurationManager.AppSettings("acsHostname")

		'the key for which the service was signed...
		Private _trustedSigningKey As String = ConfigurationManager.AppSettings("trustedSigningKey")

		'the URL that was setup in the rely party in azure
		Private _trustedAudience As String = ConfigurationManager.AppSettings("trustedAudience")

		'setting to allow the metadata to be shown
		Private _enableMetaData As Boolean = Convert.ToBoolean(ConfigurationManager.AppSettings("enableMetadata"))

		'setting to disable or enable the security module
		Private _disableSecurity As Boolean = Convert.ToBoolean(ConfigurationManager.AppSettings("disableSecurity"))

		Private Const _metaData As String = "$metadata"

		Private Sub Dispose() Implements IHttpModule.Dispose
		End Sub

		Private Sub Init(ByVal context As HttpApplication) Implements IHttpModule.Init
			AddHandler context.AuthenticateRequest, AddressOf context_BeginRequest
		End Sub

		Private Sub context_BeginRequest(ByVal sender As Object, ByVal e As EventArgs)
			If Not _disableSecurity Then
				Dim tempAcceptableURLs As String = String.Empty

				'check if the audiencename has trailing slash
				tempAcceptableURLs = _trustedAudience.ToLower
				If tempAcceptableURLs.Substring(_trustedAudience.Length - 1, 1) = "/" Then
					tempAcceptableURLs = tempAcceptableURLs.Substring(0, _trustedAudience.Length - 1)
				End If

				'first check if the person is requesting the wsdl or .svc
				If _enableMetaData <> False AndAlso HttpContext.Current.Request.Url.AbsoluteUri.ToLower <> tempAcceptableURLs AndAlso HttpContext.Current.Request.Url.AbsoluteUri.ToLower <> tempAcceptableURLs & _metaData AndAlso HttpContext.Current.Request.Url.AbsoluteUri.ToLower <> tempAcceptableURLs & "/" AndAlso HttpContext.Current.Request.Url.AbsoluteUri.ToLower <> tempAcceptableURLs & "/" & _metaData Then
					' get the authorization header
					Dim headerValue As String = HttpContext.Current.Request.Headers.Get("Authorization")

					' check that a value is there
					If String.IsNullOrEmpty(headerValue) Then
						Throw New ApplicationException("unauthorized-1.1")
					End If

					' check that it starts with 'WRAP'
					If Not headerValue.StartsWith("WRAP ") Then
						Throw New ApplicationException("unauthorized-1.2")
					End If

					Dim nameValuePair() As String = headerValue.Substring("WRAP ".Length).Split(New Char() { "="c }, 2)

					If nameValuePair.Length <> 2 OrElse nameValuePair(0) <> "access_token" OrElse (Not nameValuePair(1).StartsWith("""")) OrElse (Not nameValuePair(1).EndsWith("""")) Then
						Throw New ApplicationException("unauthorized-1.3")
					End If

					' trim off the leading and trailing double-quotes
					Dim token As String = nameValuePair(1).Substring(1, nameValuePair(1).Length - 2)

					' create a token validator
					Dim validator As New TokenValidator(Me._acsHostName, Me._serviceNamespace, Me._trustedAudience, Me._trustedSigningKey)

					' validate the token
					If Not validator.Validate(token) Then
						Dim tokenValues As Dictionary(Of String, String) = validator.GetNameValues(token)

                        Dim audienceValue As String = String.Empty
						tokenValues.TryGetValue(Me._trustedAudience, audienceValue)
						Throw New ApplicationException("unauthorized-1.4: " & _acsHostName & " - _trustedAudience: " & _trustedAudience & " -audienceValue: " & audienceValue & " - Token: " & HttpUtility.UrlDecode(token))
					End If
				End If
			End If
		End Sub
	End Class
End Namespace
