Option Strict Off

Imports System.Collections.Specialized
Imports System.Configuration
Imports System.Data.Services.Client
Imports System.Net
Imports System.Text
Imports System.Web.Mvc

Namespace SecureoDataTestBench.Controllers
	Public Class HomeController
		Inherits Controller

		Private Shared _accessControlHostName As String = ConfigurationManager.AppSettings.Get("AccessControlHostName")
		Private Shared _accessControlNamespace As String = ConfigurationManager.AppSettings.Get("AccessControlNamespace")
		Private Shared _rootURL As String = ConfigurationManager.AppSettings.Get("ServiceURL")

		'token to be placed in the request header for requests
		Private _token As String = String.Empty

		''' <summary>
		''' To handle the get request for the Azure Secure oData Service
		''' </summary>
		''' <returns>Action Result - View</returns>
		Public Function Index() As ActionResult
			ViewBag.Message = "Welcome to the Azure Secure oData Security Test Bench"

			'first thing first we need to obtain a token from Azure...
			_token = GetTokenFromACS(_rootURL & "NorthwindService.svc")

			'now that we have a valid token we can call the service as needed...
			Dim uri As New Uri(_rootURL & "NorthwindService.svc/")
			Try
				Dim northwindEntities = New oDataServiceClient.NorthwindEntities(uri)

				'add the event handler to send the token in our request header
				AddHandler northwindEntities.SendingRequest, AddressOf OnSendingRequest

				'add new data...
				Dim category = oDataServiceClient.Category.CreateCategory(0, "New cateogory")
				northwindEntities.AddToCategories(category)
				northwindEntities.SaveChanges()

				'retrieve the data...
				'sameple selecting data out...
				Dim categoiesFound = From categories In northwindEntities.Categories
				                     Select categories

				ViewBag.Categories = categoiesFound
			Catch e As DataServiceRequestException
				' trap any data service exceptions such as a security error
				'in the event that the security does not allow an insert you will receive a forbidden return
				'....
			End Try

			Return View()
		End Function

		''' <summary>
		''' An intercept on the request that adds in the token to the request header
		''' to allow authorization into the oData WCF Service
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub OnSendingRequest(ByVal sender As Object, ByVal e As SendingRequestEventArgs)
			' Add an Authorization header that contains an OAuth WRAP access token to the request.
			e.RequestHeaders.Add("Authorization", "WRAP access_token=""" & _token & """")
		End Sub

		''' <summary>
		''' Retrieve Token from Azure's Access Control Service
		''' </summary>
		''' <param name="scope">scope or audience that is seutp in Azure</param>
		''' <returns>string - token from Azure's ACS</returns>
		Private Shared Function GetTokenFromACS(ByVal scope As String) As String
            'Dim wrapPassword As String = "[YourPassword]"  // ConfigurationManager.AppSettings.Get("WrapPassword")
            'Dim wrapUsername As String = "[YourUserName]"  //ConfigurationManager.AppSettings.Get("WrapUsername")
			Dim wrapPassword As String = ConfigurationManager.AppSettings.Get("WrapPassword")
			Dim wrapUsername As String = ConfigurationManager.AppSettings.Get("WrapUsername")
			Dim response As String = String.Empty

			' request a token from ACS
			Using client As New WebClient
				client.BaseAddress = String.Format("https://{0}.{1}", _accessControlNamespace, _accessControlHostName)

				Dim values As New NameValueCollection
				values.Add("wrap_name", wrapUsername)
				values.Add("wrap_password", wrapPassword)
				values.Add("wrap_scope", scope)

				Dim responseBytes() As Byte = client.UploadValues("WRAPv0.9/", "POST", values)

				response = Encoding.UTF8.GetString(responseBytes)

				Console.WriteLine(vbCrLf & "received token from ACS: {0}" & vbCrLf, response)
			End Using

			Return HttpUtility.UrlDecode(response.Split("&"c).Single(Function(value) value.StartsWith("wrap_access_token=", StringComparison.OrdinalIgnoreCase)).Split("="c)(1))
		End Function

		''' <summary>
		''' To process the Index Request for the AJAX Sample
		''' </summary>
		''' <remarks>Note: For this example we first must retrieve the token that is then pass
		''' along to the client-side script that allows us to call the service via AJAX</remarks>
		''' <returns>ActionResult - View</returns>
		Public Function IndexAJAX() As ActionResult
			ViewBag.Message = "Welcome to the Azure Secure oData AJAX Security Test Bench"

			'first thing first we need to obtain a token from Azure...
			ViewBag.token = GetTokenFromACS(_rootURL & "NorthwindService.svc")

			Return View()
		End Function
	End Class
End Namespace
